//     Author: Maurizio Loreti, aka MLO or (HAM) I3NOO
//     Work:   University of Padova - Department of Physics
//             Via F. Marzolo, 8 - 35131 PADOVA - Italy
//     Phone:  +39 (049) 827-7216   FAX: +39 (049) 827-7102
//     EMail:  loreti@pd.infn.it
//     WWW:    http://www.pd.infn.it/~loreti/mlo.html
//
// -------------------------------------------------------------------
// Definitions of the methods declared in "dye.hh"
// -------------------------------------------------------------------
// $Id: dye.cxx,v 1.3 2005/06/14 08:57:34 loreti Exp $
// -------------------------------------------------------------------

#include <stdexcept>
#include "dye.hh"

void dye::seq_append(
  code c
) {

  // A service routine, that appends to the internal sequence (stored
  // in "_seq") the code corresponding to the style identified by "c".
  
  switch (c) {
    case reset:      _seq.push_back('0');                        break;
    case bold:       _seq.push_back('1');                        break;
    case underline:  _seq.push_back('4');                        break;
    case blink:      _seq.push_back('5');                        break;
    case reverse:    _seq.push_back('7');                        break;
    case black:      _seq.push_back('3');  _seq.push_back('0');  break;
    case red:        _seq.push_back('3');  _seq.push_back('1');  break;
    case green:      _seq.push_back('3');  _seq.push_back('2');  break;
    case yellow:     _seq.push_back('3');  _seq.push_back('3');  break;
    case blue:       _seq.push_back('3');  _seq.push_back('4');  break;
    case magenta:    _seq.push_back('3');  _seq.push_back('5');  break;
    case cyan:       _seq.push_back('3');  _seq.push_back('6');  break;
    case white:      _seq.push_back('3');  _seq.push_back('7');  break;
    case onblack:    _seq.push_back('4');  _seq.push_back('0');  break;
    case onred:      _seq.push_back('4');  _seq.push_back('1');  break;
    case ongreen:    _seq.push_back('4');  _seq.push_back('2');  break;
    case onyellow:   _seq.push_back('4');  _seq.push_back('3');  break;
    case onblue:     _seq.push_back('4');  _seq.push_back('4');  break;
    case onmagenta:  _seq.push_back('4');  _seq.push_back('5');  break;
    case oncyan:     _seq.push_back('4');  _seq.push_back('6');  break;
    case onwhite:    _seq.push_back('4');  _seq.push_back('7');  break;

    default:          throw std::runtime_error("[dye] invalid code");
  }
}

dye::dye(
  code c0,
  code c1,
  ...
) : _seq("\033[") {

  // Constructor (several actions)

  va_list vl;
  int     i;

  seq_append(c0);
  _seq.push_back(';');
  va_start(vl, c1);

  for (seq_append(c1);   (i = va_arg(vl, int)) != end;   ) {
    _seq.push_back(';');
    seq_append(static_cast<code>(i));
  }
  _seq.push_back('m');
  va_end(vl);
#ifdef DEBUG
  seq_out();
#endif // DEBUG
}

#ifdef DEBUG
void dye::seq_out(
  std::ostream & o              // Defaults to std::cerr
) {
  o << "[dye] _seq = \"<ESC>" << _seq.substr(1) << '\"' << std::endl;
}
#endif // DEBUG
